@@ -91,11 +91,22 @@ void PocoDatabase::Connect () throw (Xception)
 	// if the callback doesn't throw an exception we assume that we
 	// have a schema now, so we go one here
 
+	// check the purpose column in TCollections
+	try
+	{
+		(*m_pSession) << "select purpose from TCollections;", now;
+	}
+	catch (...)
+	{
+		// no purpose column, react
+		NoPurposeColumnCallback();
+	}
+
 	// get collection meta informations
 	try
 	{
 		Statement select(*m_pSession);
-		select << "select name,type,tbl_name from TCollections;";
+		select << "select name,type,tbl_name,purpose from TCollections;";
 		select.execute();
 
 		RecordSet rs(select);
@@ -106,6 +117,7 @@ void PocoDatabase::Connect () throw (Xception)
 				cmi.m_collection = rs.value(0).convert<string>();
 				cmi.m_type = rs.value(1).convert<string>()[0];
 				cmi.m_tblname = rs.value(2).convert<string>();
+				cmi.m_purpose = rs.value(3).convert<string>();
 				m_collectionMetaInfos.insert(cmi);
 			} while (rs.moveNext());
 		}
@@ -118,25 +130,13 @@ void PocoDatabase::Connect () throw (Xception)
 
 const PocoDatabase::CollectionMetaInfo& PocoDatabase::GetCollectionMetaInfo (const string& name) const throw (Xception)
 {
-	CollectionMetaInfo cmi = {name, '\0', ""};
+	CollectionMetaInfo cmi = {name, '\0', "", ""};
 	CollectionMetaInfos::const_iterator I = m_collectionMetaInfos.find(cmi);
 	if (I == m_collectionMetaInfos.end())
 		throw Xception("unknown collection name");
 	return *I;
 }
 
-char PocoDatabase::GetCollectionType (const string& name) const throw (Xception)
-{
-	try
-	{
-		return GetCollectionMetaInfo(name).m_type;
-	}
-	catch (...)
-	{
-		throw Xception("could not determine collection type");
-	}
-}
-
 //=== DBTransactor =========================================================
 DBTransactor::DBTransactor (Poco::Data::Session* pSession, size_t& counter)
 	: m_pSession(pSession)
