@@ -86,7 +86,7 @@ void PocoDatabaseImpl::ListCollections () throw (Xception)
 		}
 		string operator () (int idx)
 		{
-			static const size_t widths[] = {6, 8, 6, 7, 20}; // real column widths
+			static const size_t widths[] = {6, 8, 6, 7, 20, 20}; // real column widths
 			size_t width = widths[idx];
 			string result(m_s);
 			while (result.length() < width)
@@ -99,7 +99,7 @@ void PocoDatabaseImpl::ListCollections () throw (Xception)
 
 	try
 	{
-		cout << "  " << fill("name")(0) << fill("type")(1) << fill("table")(2) << fill("recs")(3) << fill("first")(4) << "last" << endl;
+		cout << "  " << fill("name")(0) << fill("type")(1) << fill("table")(2) << fill("recs")(3) << fill("first")(4) << fill("last")(5) << "purpose" << endl;
 
 		foreach (const CollectionMetaInfo& cmi, m_collectionMetaInfos)
 		{
@@ -118,7 +118,9 @@ void PocoDatabaseImpl::ListCollections () throw (Xception)
 			int count;
 			string min, max;
 			(*m_pSession) << "select count(*),min(t),max(t) from " << cmi.m_tblname << ";", into(count), into(min), into(max), now;
-			cout << fill(lexical_cast<string>(count))(3) << fill(min)(4) << max << endl;
+			cout << fill(lexical_cast<string>(count))(3) << fill(min)(4) << fill(max)(5);
+
+			cout << cmi.m_purpose << endl;
 		}
 	}
 	catch (...)
@@ -126,45 +128,62 @@ void PocoDatabaseImpl::ListCollections () throw (Xception)
 	}
 }
 
-void PocoDatabaseImpl::AddCollection (const string& name) throw (Xception) // "name[, n|r|t]"
+void PocoDatabaseImpl::AddCollection (const string& name) throw (Xception) // "name[, n|r|t[, purpose]]"
 {
-	if (m_verbose)
-		cout << "adding collection" << endl;
-
 	try
 	{
-		// split collection name and type
-		string n, t;
+		// split collection name, type and purpose
+		string n, t, p;
 		{
-			regex rx("\\s*([^,\\s]+)(?:\\s*,\\s*([nrt])\\s*)?");
+			regex rx("\\s*([^,\\s]+)(?:\\s*,\\s*([nrt])\\s*(?:,(.+))?)?");
 			smatch mr;
 			if (!regex_match(name, mr, rx))
-				throw Xception(format("illegal collection name: %s") % name);
+				throw Xception(format("illegal collection specification: %s") % name);
 			n = mr[1];
 			t = mr[2];
 			if (t.empty())
 				t = "n"; // numeric
+			p = mr[3];
+			trim(p);
 		}
 
 		// check for existing collection
 		try
 		{
-			GetCollectionMetaInfo(n);
-			throw Xception(format("collection already exists: %s") % n);
+			const CollectionMetaInfo& cmi = GetCollectionMetaInfo(n);
+			if (cmi.m_type == t[0])
+			{
+				if (cmi.m_purpose != p)
+				{
+					UpdatePurpose(n, p);
+					const_cast<CollectionMetaInfo&>(cmi).m_purpose = p;
+				}
+				return;
+			}
+			else
+				throw Xception(format("collection %s already exists but differs in type: %c <-> %s") % n % cmi.m_type % t);
 		}
 		catch (...)
 		{
-			// ok, table doesn't exist
+			// ok, collection doesn't exist
+		}
+
+		if (m_verbose)
+		{
+			cout << "adding collection " << n;
+			if (!p.empty())
+				cout << " (" << p << ')';
+			cout << endl;
 		}
 
 		// build new table name
 		const string& tblname = GetNewTblname();
 
 		// do the database specific SQL job
-		CreateCollectionInSchema(n, t[0], tblname);
+		CreateCollectionInSchema(n, t[0], tblname, p);
 
 		// create a CollectionMetaInfo
-		CollectionMetaInfo cmi = {n, t[0], tblname};
+		CollectionMetaInfo cmi = {n, t[0], tblname, p};
 		m_collectionMetaInfos.insert(cmi);
 	}
 	catch (...)
@@ -221,6 +240,7 @@ void PocoDatabaseImpl::DeleteAllCollections () throw (Xception)
 		}
 
 		(*m_pSession) << "delete from TCollections where name not in ('*','#');", now;
+		(*m_pSession) << "update TCollections set purpose = NULL;", now;
 		(*m_pSession) << "delete from C0;", now;
 		(*m_pSession) << "delete from C1;", now;
 
@@ -285,7 +305,20 @@ void PocoDatabaseImpl::AddRejected (const ptime& timestamp, const string& expr)
 	}
 	catch (...)
 	{
-		throw Xception("could not insert expression");
+		throw Xception("could not insert rejection");
+	}
+}
+
+void PocoDatabaseImpl::DeleteRejected (const ptime& timestamp) throw (Xception)
+{
+	try
+	{
+		const string& t = lexical_cast<string, ptime>(timestamp);
+		(*m_pSession) << "delete from TRejected where t='" << t << "';", now;
+	}
+	catch (...)
+	{
+		throw Xception("could not delete rejection");
 	}
 }
 
@@ -306,7 +339,12 @@ void PocoDatabaseImpl::DeleteAllRejections () throw (Xception)
 
 char PocoDatabaseImpl::GetCollectionType (const string& name) const throw (Xception)
 {
-	return PocoDatabase::GetCollectionType(name);
+	return GetCollectionMetaInfo(name).m_type;
+}
+
+string PocoDatabaseImpl::GetCollectionPurpose (const string& name) const throw (Xception)
+{
+	return GetCollectionMetaInfo(name).m_purpose;
 }
 
 void PocoDatabaseImpl::GetCollections (Collections& collections) const throw (Xception)
@@ -351,7 +389,7 @@ void PocoDatabaseImpl::GetCollectionItems (const string& name, CollectionItems&
 	}
 	catch (...)
 	{
-		throw Xception("could not get collection items");
+		throw Xception("could not get collection data");
 	}
 }
 
@@ -363,33 +401,14 @@ void PocoDatabaseImpl::AddCollectionsItems (const CollectionsItems& items) throw
 
 		foreach (const CollectionsItems::value_type& vt, items)
 		{
-			const CollectionMetaInfo* pCmi = NULL;
-			try
-			{
-				 pCmi = &GetCollectionMetaInfo(vt.first);
-			}
-			catch (const Xception& )
-			{
-				// if the collection doesn't exist ...
-				// ... create it now
-				const string& n = vt.first;
-				char t = GetCollectionTypeFromValue(vt.second.second);
-				const string& tblname = GetNewTblname();
-
-				CreateCollectionInSchema(n, t, tblname);
-
-				CollectionMetaInfo cmi = {n, t, tblname};
-				CollectionMetaInfos::iterator I = m_collectionMetaInfos.insert(cmi).first;
-				pCmi = &(*I);
-			}
-			InsertOrUpdateCollectionItem(*pCmi, vt.second);
+			InsertOrUpdateCollectionItem(GetCollectionMetaInfo(vt.first), vt.second);
 		}
 
 		transactor.Commit();
 	}
 	catch (...)
 	{
-		throw Xception("could not add collection items");
+		throw Xception("could not add collection data");
 	}
 }
 
@@ -410,18 +429,18 @@ void PocoDatabaseImpl::DeleteCollectionsItems (const ptime& timestamp, const Col
 	}
 	catch (...)
 	{
-		throw Xception("could not delete data");
+		throw Xception("could not delete collection data");
 	}
 }
 
-void PocoDatabaseImpl::GenerateExpressions (Expressions& expressions, const ptime& timestamp) const throw (Xception)
+void PocoDatabaseImpl::GenerateExpressions (Expressions& expressions, const ptime& timestamp, const string& rejected) const throw (Xception)
 {
 	try
 	{
 		typedef multimap<ptime, pair<any, string> > Data;
 
+		// collect data from the collections
 		Data data;
-		size_t n = 0;
 		{
 			foreach (const CollectionMetaInfo& cmi, m_collectionMetaInfos)
 			{
@@ -458,10 +477,10 @@ void PocoDatabaseImpl::GenerateExpressions (Expressions& expressions, const ptim
 						data.insert(Data::value_type(lexical_cast<ptime>(rs.value(0).convert<string>()), value));
 					} while (rs.moveNext());
 				}
-				n++;
 			}
 		}
 
+		// build expressions from collection data
 		Data::const_iterator I = data.begin();
 		while (I != data.end())
 		{
@@ -505,6 +524,31 @@ void PocoDatabaseImpl::GenerateExpressions (Expressions& expressions, const ptim
 
 			expressions.insert(Database::Expressions::value_type(timestamp, expr));
 		}
+
+		// collect data also from rejections
+		{
+			// build the SQL string
+			string sql("select t,expr from TRejected");
+			if (timestamp != not_a_date_time)
+				sql += " where t='" + lexical_cast<string>(timestamp) + "'";
+			sql += " order by 1;";
+
+			// execute
+			Statement select(*m_pSession);
+			select << sql;
+			select.execute();
+
+			// fetch data
+			RecordSet rs(select);
+			if (rs.moveFirst())
+			{
+				do {
+					const ptime& timestamp = lexical_cast<ptime>(rs.value(0).convert<string>());
+					const string& expr = rejected + rs.value(1).convert<string>();
+					expressions.insert(Database::Expressions::value_type(timestamp, expr));
+				} while (rs.moveNext());
+			}
+		}
 	}
 	catch (...)
 	{
