@@ -34,7 +34,111 @@ InputImpl::InputImpl (const string& option_key)
 {
 }
 
-void InputImpl::Parse (const string& expr, const ptime& timestamp, bool verbose, Database::CollectionsItems& items) const throw (Xception)
+string InputImpl::ConvertMailSubject (const string& subject, InputImpl::decode_quoted_printable_proc decode_quoted_printable, InputImpl::convert_charset_proc convert_charset)
+{
+	// a coded mail subject can be a complex thing, it can have a shape
+	// like this
+	//
+	//	coded ascii coded coded coded ascii coded
+	//
+	// "coded" means an expression like this
+	//
+	//	=?charset?Q?text?=
+	//
+	// "charset" is a specification like "iso-8859-1",
+	// "text" is a character string in quoted printable notation
+	//
+	// for every of these expressions we must first decode the quoted
+	// printable stuff into human readable text and then convert it's
+	// character set into utf-8
+
+	string result;
+	{
+		static const regex rx_coded_part("=\\?(.+)\\?Q\\?(.+)\\?=");
+
+		string s(subject);
+		while (!s.empty())
+		{
+			smatch mr;
+			if (regex_search(s, mr, rx_coded_part, match_any))
+			{
+				result += string(s, 0, mr.position());
+
+				string text(mr[2]);
+
+				// if a coded text contains underlines
+				// these underlines mean spaces, real
+				// underlines are coded
+				for (size_t i = 0; i < text.length(); i++)
+				{
+					if (text[i] == '_')
+						text[i] = ' ';
+				}
+
+				decode_quoted_printable(text);
+
+				string charset(mr[1]);
+				if (charset.empty())
+					charset = "us-ascii";
+				else
+					to_lower(charset);
+				s.erase(0, mr.length());
+
+				if (charset != "utf-8")
+					convert_charset(text, charset);
+
+				// damned, some guys even code spaces!
+				if (text == " ")
+				{
+					trim_right(result);
+					trim_left(s);
+				}
+
+				result += text;
+			}
+			else
+			{
+				result += s;
+				s.clear();
+			}
+		}
+	}
+
+	return result;
+}
+
+string InputImpl::ExtractCharsetFromContentType (const string& content_type)
+{
+	// a content type is normally something like this:
+	//
+	//	text/plain; charset=iso-8859-15
+	//
+	// the part in front of the semicolon defines the type of the
+	// content, we only accept text/plain here be definition, the
+	// other part defines the character set of the text 
+	//
+	// some mail clients insert newlines after the semicolon or
+	// spaces or tabs at some places in this string, so we have to be
+	// very robust here
+
+	string s(content_type);
+	to_lower(s);
+
+	for (size_t i = 0; i < s.length(); i++)
+	{
+		if (s[i] < 0x20)
+			s[i] = 0x20;
+	}
+
+	static const regex rx_charset("^text/plain\\s*;\\s*charset\\s*=\\s*(.+)\\s*$");
+	smatch mr;
+	if (regex_match(s, mr, rx_charset, boost::match_not_dot_newline))
+		return string(mr[1]);
+	else
+		return string(); // no charset or not text/plain
+}
+
+void InputImpl::Parse (const string& expr, const ptime& timestamp, bool verbose, Database::CollectionsItems& items) throw (Xception)
 {
 	string line(expr);
 
