@@ -41,6 +41,8 @@ InputInteractive::InputInteractive (const string& option_key)
 {
 }
 
+static const string s_rejected("!!! rejected !!! ");
+
 #ifdef USE_READLINE
 	static string s_expr;
 
@@ -98,7 +100,7 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 
 	// do a database over all query
 	Database::Expressions expressions;
-	database.GenerateExpressions(expressions, not_a_date_time);
+	database.GenerateExpressions(expressions, not_a_date_time, s_rejected);
 
 	Database::Expressions::iterator I = expressions.end();
 	if (!expressions.empty())
@@ -121,7 +123,11 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 
 #ifdef USE_READLINE
 		if (I != expressions.end())
+		{
 			s_expr = (*I).second;
+			if (s_expr.find(s_rejected) == 0)
+				s_expr.erase(0, s_rejected.length());
+		}
 		char* buf = readline(prompt.c_str());
 		if (!buf)
 		{
@@ -216,44 +222,58 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 					try
 					{
 						const ptime& timestamp = (*I).first;
-						bool delete_current_line = true;
-						if (line.empty())
-							database.DeleteCollectionsItems(timestamp, Database::Collections());
+						const string& expr = (*I).second;
+						if (expr.find(s_rejected) == 0)
+						{
+							if (!line.empty())
+							{
+								cout	<< "deletion of parts of rejections not supported" << endl
+									<< "use D to delete the entire rejection or just modify the string" << endl;
+							}
+							else
+								database.DeleteRejected(timestamp);
+						}
 						else
 						{
-							Database::Collections collections;
-							typedef tokenizer<boost::char_separator<char> > tokenizer;
-							tokenizer tok(line);
-							for (tokenizer::iterator J = tok.begin(); J != tok.end(); J++)
-							{
-								string collection(*J);
-								if (collection == ";")
-									collection = '#';
-								collections.insert(collection);
-							}
-							database.DeleteCollectionsItems(timestamp, collections);
-							Database::Expressions temp;
-							database.GenerateExpressions(temp, timestamp);
-							if (temp.size() == 1)
-							{
-								(*I).second = (*temp.begin()).second;
-								delete_current_line = false;
-							}
-						}
-
-						if (delete_current_line)
-						{
-							Database::Expressions::iterator J = I;
-							ptime t(not_a_date_time);
-							if (J != expressions.begin())
-								t = (*--J).first;
+							bool delete_current_line = true;
+							if (line.empty())
+								database.DeleteCollectionsItems(timestamp, Database::Collections());
 							else
 							{
-								if (J != expressions.end())
-									t = (*++J).first;
+								Database::Collections collections;
+								typedef tokenizer<boost::char_separator<char> > tokenizer;
+								tokenizer tok(line);
+								for (tokenizer::iterator J = tok.begin(); J != tok.end(); J++)
+								{
+									string collection(*J);
+									if (collection == ";")
+										collection = '#';
+									collections.insert(collection);
+								}
+								database.DeleteCollectionsItems(timestamp, collections);
+								Database::Expressions temp;
+								database.GenerateExpressions(temp, timestamp, "");
+								if (temp.size() == 1)
+								{
+									(*I).second = (*temp.begin()).second;
+									delete_current_line = false;
+								}
+							}
+
+							if (delete_current_line)
+							{
+								Database::Expressions::iterator J = I;
+								ptime t(not_a_date_time);
+								if (J != expressions.begin())
+									t = (*--J).first;
+								else
+								{
+									if (J != expressions.end())
+										t = (*++J).first;
+								}
+								expressions.erase(I);
+								I = (t == not_a_date_time) ? expressions.end() : expressions.find(t);
 							}
-							expressions.erase(I);
-							I = (t == not_a_date_time) ? expressions.end() : expressions.find(t);
 						}
 					}
 					catch (Xception& e)
@@ -272,13 +292,20 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 						try
 						{
 							const ptime& timestamp = (*I).first;
+							const string expr((*I).second);
+
 							Database::CollectionsItems items;
 							Parse(line, timestamp, false, items);
 							database.AddCollectionsItems(items);
 							Database::Expressions temp;
-							database.GenerateExpressions(temp, timestamp);
+							database.GenerateExpressions(temp, timestamp, "");
 							if (temp.size() == 1)
 								(*I).second = (*temp.begin()).second;
+
+							// in case of a former rejected line,
+							// delete the rejection now
+							if (expr.find(s_rejected) == 0)
+								database.DeleteRejected(timestamp);
 						}
 						catch (Xception& e)
 						{
