@@ -31,6 +31,7 @@ using namespace boost::program_options;
 #include <Poco/Net/NetException.h>
 #include <Poco/Net/POP3ClientSession.h>
 #include <Poco/Net/MailMessage.h>
+#include <Poco/Net/QuotedPrintableDecoder.h>
 
 using namespace Poco;
 using namespace Poco::Net;
@@ -41,13 +42,27 @@ Pop3MailClient::Pop3MailClient (const string& option_key)
 {
 }
 
+	static void decode_quoted_printable (string& s)
+	{
+		stringstream ss(s);
+		QuotedPrintableDecoder decoder(ss);
+		string decoded;
+		decoder >> decoded;
+		s = decoded;
+	}
+
+	static void convert_charset (string& s, const string& from_charset)
+	{
+		s = ConvertFrom(s, GetEncoding(from_charset));
+	}
+
 void Pop3MailClient::Do (const Config& config, Database& database) const throw (Xception)
 {
 	try
 	{
 		bool verbose = config.GetBoolOption("verbose");
 		if (verbose)
-			cout << "looking for mail" << endl;
+			cout << "looking for mail (POP3)" << endl;
 
 		// get configuration data
 		const string& server = config.GetStringOption(m_option_key + ".server");
@@ -82,11 +97,9 @@ void Pop3MailClient::Do (const Config& config, Database& database) const throw (
 			string s(message["Subject"]);
 			{
 				trim(s);
-				static const regex rx_coded_subject("^=\\?.+\\?Q\\?(.+)\\?=$");
-				smatch mr;
-				if (regex_match(s, mr, rx_coded_subject, boost::match_not_dot_newline))
-					s = mr[1];
+				s = InputImpl::ConvertMailSubject(s, decode_quoted_printable, convert_charset);
 			}
+
 			if (s != subject)
 				continue;
 
@@ -111,14 +124,7 @@ void Pop3MailClient::Do (const Config& config, Database& database) const throw (
 			}
 
 			// get message body and decode it correctly (this means make it UTF-8)
-			string charset;
-			{
-				const string& content_type = message.getContentType();
-				static const regex rx_charset("charset=([^;]+)");
-				smatch mr;
-				if (regex_search(content_type, mr, rx_charset, boost::match_not_dot_newline))
-					charset = mr[1];
-			}
+			const string& charset = ExtractCharsetFromContentType(message.getContentType());
 			stringstream ss;
 			if (charset.empty())
 				ss << message.getContent(); // should be 7bit ASCII
@@ -129,6 +135,10 @@ void Pop3MailClient::Do (const Config& config, Database& database) const throw (
 			string line;
 			while (getline(ss, line))
 			{
+				trim(line);
+				if (line.empty())
+					continue;
+
 				try
 				{
 					Database::CollectionsItems items;
