@@ -78,6 +78,7 @@ void OutputXML::Do (const Config& config, const Database& database) const throw
 	foreach (const string& collection, collections)
 	{
 		char type = database.GetCollectionType(collection);
+
 		ofs << INDENT << "<collection name=\"" << collection << "\" type=\"";
 		switch (type)
 		{
@@ -86,7 +87,7 @@ void OutputXML::Do (const Config& config, const Database& database) const throw
 			case 't':	ofs << "text"; break;
 			default:	break;
 		}
-		ofs << "\">" << endl;
+		ofs << "\" purpose=\"" << database.GetCollectionPurpose(collection) << "\">" << endl;
 
 		indent++;
 		Database::CollectionItems items;
